<?php
/*
 * spkrWatchdog.php is to be used for the following purposes:
 * 1) provides a 'scanning LED' output that occurs when the file 'cylon' exists in the state folder.
 * 2) provides a 'flashing leds' output that occurs when the file 'flash' exists in the state folder.
 * 3) pings the katoombasound IP address once every ten minutes to keep the network alive.
 * 4) TBD - it monitors critical functions and if they fail for an extended period (2 minutes?) then the device is returned to safe code and default settings
 *
 * In order to ensure maximum reliability, spkrWatchdog.php does not rely on other PHP include files or external data (settings etc).
 * And, it is never touched by the user operations.
 * That is also why the ping keep alive IP address is hard coded and the periodicity is also hard coded.
 * If you do decide to make the keep alive IP and periodicity user configurable, make sure you revert to sane defaults if the settings file cannot be accessed for any reason.
*/

// todo - the web service monitor fills up the logs when in access point mode. Fix that.

echo date("Y-m-d H:i:s").": spkrWatchdog service started.\n";
echo date("Y-m-d H:i:s").": Checking the i2c bus now.\n";

// We want to try to rescue the i2cbus to get it going again since this script exits if it cannot access i2c bus 0 or 1. Add reset code right here.
exec('i2cget -y 0 0x21', $output, $returned);
if(strlen($returned)>0) $i2cbus='0';
else{
    exec('i2cget -y 1 0x21', $output, $returned);
    if(strlen($returned)>0) $i2cbus='1';
    else {
        echo date("Y-m-d H:i:s").": ERROR: spkrWatchdog.php could not access i2c bus. Sleeping 10 seconds.\n";
        sleep(10);
        exit;
    }
}
echo date("Y-m-d H:i:s").": Setting i2c IO port pins to pull ups.\n";

// set IO port LED pins to pull ups
exec('i2cset -y '.$i2cbus.' 0x22 0x06 0xff');
exec('i2cset -y '.$i2cbus.' 0x21 0x06 0xff');
exec('i2cset -y '.$i2cbus.' 0x22 0x00 0x55');
exec('i2cset -y '.$i2cbus.' 0x21 0x00 0x55');
exec('i2cset -y '.$i2cbus.' 0x24 0x00 0x00');

echo date("Y-m-d H:i:s").": Going into main endless loop.\n";

$speed=25000;
$pingTime=60;
$lastPingTime=0;
$lastDebug=0;
$webServer=-1;

// and then continue into main...
while(1){
    while(file_exists("/var/www/html/state/cylon")){
        for($count=0;$count<8;$count++){
            $mask=(1<<$count)^0xfd;
            exec('i2cset -y '.$i2cbus.' 0x22 0x09 '.$mask);
            usleep($speed);
        }
        for($count=0;$count<7;$count++){
            $mask=(1<<$count)^0xff;
            exec('i2cset -y '.$i2cbus.' 0x21 0x09 '.$mask);
            usleep($speed);
        }
        for($count=7;$count>=0;$count--){
            $mask=(1<<$count)^0xff;
        exec('i2cset -y '.$i2cbus.' 0x21 0x09 '.$mask);
            usleep($speed);
        }
        for($count=6;$count>=0;$count--){
            $mask=(1<<$count)^0xfd;
            exec('i2cset -y '.$i2cbus.' 0x22 0x09 '.$mask);
            usleep($speed);
        }
    }
    while(file_exists("/var/www/html/state/flash")){
        // leds on
        exec('i2cset -y '.$i2cbus.' 0x22 0x09 0x02');
        exec('i2cset -y '.$i2cbus.' 0x21 0x09 0x00');
        usleep(500000);
        // leds off
        exec('i2cset -y '.$i2cbus.' 0x22 0x09 0xA8');
        exec('i2cset -y '.$i2cbus.' 0x21 0x09 0xAA');
        usleep(500000);

        $counter = file_get_contents("/var/www/html/state/flash");
        if(is_numeric($counter--)){
            if($counter>0) file_put_contents("/var/www/html/state/flash",$counter);
            else unlink("/var/www/html/state/flash");
        } else unlink("/var/www/html/state/flash");
    }
    // do the apache keep alive log
    $now=time();
    if($now-$pingTime>=$lastPingTime){
        $lastPingTime=$now;
        exec('ping -c1 69.163.177.231 > /dev/null 2>&1');
        unset($results);
        exec('wget -q -O /dev/null localhost; echo $?',$results);
        if($webServer!=$results[0]&&$webServer!=-1){
            if($results[0]==0) file_put_contents("/var/www/html/log/webService.log", date("Y-m-d H:i:s")." : Web Server came up. wget returned: ".$results[0]."\n", FILE_APPEND | LOCK_EX);
            else file_put_contents("/var/www/html/log/webService.log", date("Y-m-d H:i:s")." : Web Server went down. wget returned: ".$results[0]."\n", FILE_APPEND | LOCK_EX);
        }
        $webServer=$results[0];
    }
    // do the hourly debug logs (if enabled)
    $now=time();
    if(file_exists("/var/www/html/state/hourlyLogDump")){
        if($now-3600>=$lastDebug){
            echo date("Y-m-d H:i:s").": Creating hourly log dump\n";
            $lastDebug=$now;
            unset($results);
            exec("sudo uname -a",$results);
            file_put_contents("/var/www/html/log/uname.txt", implode($results));
            unset($results);

            exec("sudo journalctl -u wpa_supplicant.service",$results);
            file_put_contents("/var/www/html/log/wpa_supplicant.service.txt", preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo journalctl -u chrony.service",$results);
            file_put_contents("/var/www/html/log/chrony.service.txt",  preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo journalctl -u dhcpcd.service",$results);
            file_put_contents("/var/www/html/log/dhcpcd.service.txt",  preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo df",$results);
            file_put_contents("/var/www/html/log/df.txt", preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo journalctl -u spkrIo.service",$results);
            file_put_contents("/var/www/html/log/spkrIo.service.txt",  preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo journalctl -u spkrWatchdog.service",$results);
            file_put_contents("/var/www/html/log/spkrWatchdog.service.txt",  preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo systemctl --all",$results);
            file_put_contents("/var/www/html/log/ListAllUnits.txt",  preg_replace('/$/', "\n", $results));

            unset($results);
            exec("sudo ps -ef",$results);
            file_put_contents("/var/www/html/log/ListAllProcesses.txt",  preg_replace('/$/', "\n", $results));

            $logDumpFileName="SPKR7-All-Logs--".date("Y-m-d-H-i-s").".tar";
            exec("sudo find /var/www/html/tmp/ -type f | sort -r | tail -n +24 | xargs rm");
            exec("sudo tar -cvf /var/www/html/tmp/".$logDumpFileName." /var/www/html/log /var/log");
        }
    }else{
        $lastDebug=$now-3600;   // preset lastDebug to ensure immediate creation of log when logging is enabled
    }
    if(date("His",time())==="000000" && file_exists("/var/www/html/state/midnightRestart")){
        echo date("Y-m-d H:i:s").": Scheduled midnight restarts are enabled and it is midnight. So, a restart of the speaker switch is happening right now.\n";
        exec('sudo sync'); // sync up the file systems first
        exec('sudo shutdown -r now'); // reboot the speaker switch
    }
    sleep(1);
}
?>